"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var types_generator_1 = require("./types.generator");
var utilities_1 = require("../utilities");
var fragments_generator_1 = require("./fragments.generator");
var templates_generator_1 = require("./templates.generator");
var hooks_generator_1 = require("./hooks.generator");
var store_1 = require("../store");
exports.createMethodsArgs = function (field) {
    var prefix = store_1.ParametersStore.prefix, suffix = store_1.ParametersStore.suffix;
    var _a = field.args.reduce(function (acc, arg) {
        var argName = arg.name;
        var type = types_generator_1.generateQGLArg(arg);
        acc.GQLVariables.push("$" + argName + ": " + type);
        acc.GQLArgs.push(argName + ": $" + argName);
        return acc;
    }, {
        GQLVariables: [],
        GQLArgs: [],
    }), GQLVariables = _a.GQLVariables, GQLArgs = _a.GQLArgs;
    var methodArgsType = '';
    if (field.args.length) {
        var parsedSuffix = (suffix ? suffix : '') + 'Args';
        methodArgsType = "" + (prefix ? prefix : '') + field.name + parsedSuffix;
    }
    return {
        GQLArgs: GQLArgs,
        GQLVariables: GQLVariables,
        methodArgsType: methodArgsType,
    };
};
exports.createGraphQLFunction = function (_a) {
    var field = _a.field, type = _a.type, renderedFragmentInner = _a.renderedFragmentInner;
    var hasArgs = field.args.length > 0;
    var methodName = field.name;
    var methodArgsType = exports.createMethodsArgs(field).methodArgsType;
    var isScalar = utilities_1.evaluateType(field).isScalar;
    var returnedTypeDisplay = types_generator_1.getOneTSTypeDisplay({ field: field });
    var Query = templates_generator_1.queryBuilder({ field: field, isScalar: isScalar, renderedFragmentInner: renderedFragmentInner, type: type });
    var typeNameLower = type;
    var typeNameUpper = utilities_1.capitalizeFirstLetter(type);
    var withArgs = hasArgs
        ? utilities_1.areAllArgsOptional(field.args)
            ? 'WithOptionalArgs'
            : 'WithArgs'
        : '';
    if (isScalar) {
        return "\n    " + (field.description ? "/** " + field.description + " */" : '') + "\n    " + methodName + "(): Abortable" + typeNameUpper + withArgs + "<" + returnedTypeDisplay + (hasArgs ? ',' + methodArgsType : '') + "> {\n      const " + typeNameLower + " = " + Query + "\n      return abortable" + typeNameUpper + "(" + typeNameLower + ", " + hasArgs + ");\n        }\n    ,";
    }
    else {
        return "\n    " + (field.description ? "/** " + field.description + " */" : '') + "\n    " + methodName + "(): Fragmentable" + typeNameUpper + withArgs + "<" + returnedTypeDisplay + (hasArgs ? ',' + methodArgsType : '') + "> {\n    return {\n      $fragment: (fragment: string | DocumentNode) => {\n        const { isString, isFragment, fragmentName } = guessFragmentType(fragment);\n        const " + typeNameLower + " = " + Query + "\n        return abortable" + typeNameUpper + "(" + typeNameLower + ", " + hasArgs + ");\n      }\n    }\n      }\n  ,";
    }
};
exports.buildMethod = function (_a) {
    var field = _a.field, type = _a.type, ObjectTypes = _a.ObjectTypes, mode = _a.mode;
    var _b;
    var _c = utilities_1.evaluateType(field), isScalar = _c.isScalar, isEnum = _c.isEnum, typeName = _c.typeName;
    var renderedFragmentInner = "${isString ? fragment : '...' + fragmentName}";
    if (!isScalar && !isEnum && utilities_1.isReturnTypeEdge(ObjectTypes, typeName)) {
        renderedFragmentInner = (_b = fragments_generator_1.createConnectionFragment(typeName, ObjectTypes, renderedFragmentInner), (_b !== null && _b !== void 0 ? _b : renderedFragmentInner));
    }
    var createParams = { field: field, type: type, renderedFragmentInner: renderedFragmentInner };
    if (mode === 'template') {
        return templates_generator_1.createQueryFunction(createParams);
    }
    else if (mode === 'react-hooks') {
        return hooks_generator_1.createReactApolloHook(createParams);
    }
    else if (mode === 'vue-hooks') {
        return hooks_generator_1.createVueApolloHook(createParams);
    }
    else {
        return exports.createGraphQLFunction(createParams);
    }
};
