"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utilities_1 = require("../utilities");
var parameters_store_1 = require("../store/parameters.store");
exports.generateEnumType = function (object) {
    var prefix = parameters_store_1.ParametersStore.prefix, suffix = parameters_store_1.ParametersStore.suffix;
    var ObjectName = object.name;
    var generatedFields = object.enumValues.map(function (field) { return field.name; });
    return (object.description ? "/** " + object.description + " */\n" : '') + " export enum " + (prefix ? prefix : '') + ObjectName + (suffix ? suffix : '') + " {\n        " + generatedFields
        .map(function (enumType) { return utilities_1.capitalizeAllWord(enumType) + " = \"" + enumType + "\""; })
        .join(',\n') + "\n  }";
};
exports.getOneTSTypeDisplay = function (_a) {
    var field = _a.field;
    var prefix = parameters_store_1.ParametersStore.prefix, suffix = parameters_store_1.ParametersStore.suffix, listScalars = parameters_store_1.ParametersStore.listScalars;
    var _b = utilities_1.evaluateType(field), isScalar = _b.isScalar, typeName = _b.typeName, isArray = _b.isArray;
    var returnedName = isScalar
        ? listScalars[typeName]
        : "" + (prefix ? prefix : '') + typeName + (suffix ? suffix : '');
    return returnedName + (isArray ? '[]' : '');
};
exports.generatedTsFields = function (fields) {
    return fields.map(function (field) {
        var propertyName = field.name;
        var isOptional = utilities_1.evaluateType(field).isOptional;
        var TStypeName = exports.getOneTSTypeDisplay({ field: field });
        return (field.description ? "/** " + field.description + "*/\n" : '') + " " + propertyName + (isOptional ? '?' : '') + ": " + (isOptional ? "Maybe<" + TStypeName + ">" : TStypeName) + ";";
    });
};
exports.generateQGLArg = function (field) {
    var _a = utilities_1.evaluateType(field), isArray = _a.isArray, isArrayOptional = _a.isArrayOptional, isOptional = _a.isOptional, typeName = _a.typeName;
    return "" + (isArray ? '[' : '') + typeName + (isOptional ? '' : '!') + (isArray ? ']' : '') + (isArrayOptional ? '!' : '');
};
exports.getObjectTSInterfaces = function (object) {
    var fieldsKey = object.kind === 'OBJECT' || object.kind === 'INTERFACE' ? 'fields' : 'inputFields';
    var generatedFields = exports.generatedTsFields(object[fieldsKey]);
    return exports.buildTsInterfaceString(object, generatedFields);
};
exports.getQueriesArgsTSInterfaces = function (object) {
    var generatedFields = exports.generatedTsFields(object.args);
    return exports.buildTsInterfaceString(object, generatedFields, 'Args');
};
exports.buildTsInterfaceString = function (type, fields, additionalSuffix) {
    var prefix = parameters_store_1.ParametersStore.prefix, suffix = parameters_store_1.ParametersStore.suffix;
    return (type.description ? "/** " + type.description + " */\n" : '') + " export interface " + (prefix ? prefix : '') + type.name + (suffix ? suffix : '') + (additionalSuffix ? additionalSuffix : '') + " {\n    " + fields.join('\n') + "\n  }\n";
};
